import re

from anytree import Node

from bs4 import BeautifulSoup

import requests

def get_soup(champion: str, role: str):
    url = f"https://app.mobalytics.gg/lol/champions/{champion}/build?role={role}"
    page = requests.get(url)
    return BeautifulSoup(page.content, "html.parser")

def mobalytics_scraper(champion: str, role: str, matchup: str, verbose: bool) -> Node:
    """Scrapes a build from Mobalytics.gg."""
   
    soup = get_soup(champion, role)
    # mobalytics won't show build info for certain roles
    # when this happens, just redirect to the most popular allowed role ?
    # check if valid
    matches = soup.find_all("div", class_="css-jboygh e3vq2as0")
    not_allowed = []
    for entry in matches:
        p = entry.find("img")["alt"].lower()
        not_allowed.append(p)

    if role in not_allowed:
        print(f"Mobalytics doens't have a build for {role}. \nHere's the default build instead.")
        role = ""
        soup = get_soup(champion, role)

    c = soup.find("p", class_="css-1yvcufn eo6ba8g4").text
    r = soup.find("div", class_="css-p3pzap eo6ba8g5").text

    # create tree entries for default output
    title = f"{c} {r} from Mobalytics"
    root = Node(title)

    # get the runes
    runes = Node("Runes", parent=root)
    matches = soup.find_all("img", class_="css-1la33yl e16p94fx0")
    for entry in matches:
        r = entry["alt"]
        Node(r, parent=runes)

    # get the shards
    matches = soup.find_all("img", class_="css-1vgqbrs ed9gm2s1")
    conversion = {
        "5001": "Health",
        "5002": "Armor",
        "5003": "Magic Resist",
        "5005": "Attack Speed",
        "5007": "Ability Haste",
        "5008": "Adaptive Force",
    }
    # if verbose:
    #     shards = Node("Shards", parent=root)
    for entry in matches:
        shard_id = entry["src"].split(".png")[0][-4:]  # was blah/####.png
        shard = conversion.get(shard_id)
        # if verbose:
        #     Node(shard, parent=shards)
        # else:
        #     Node(shard, parent=runes)
        Node(shard, parent=runes)
    
    # get the build
    build = Node("Build", parent=root)

    # get time targets
    if verbose:
        time_targets = []
        matches = soup.find_all("p", class_="css-1ofmdln ehobrmq7")
        for entry in matches:
            time_targets.append(entry.text)

    # add all relevant entries
    for cycle, entry in enumerate(soup.find_all("div", class_="ednsys62 css-1taoj5l ehobrmq2")):
        # Starter items
        if cycle == 0 and verbose:
            starter = Node(f"Starter Items {time_targets[cycle]}", parent=build)
            for content in entry.contents:
                Node(re.findall(r"alt=\"(.*)\" c", str(content))[0], starter)
        # Early items
        if cycle == 1 and verbose:
            early = Node(f"Early Items {time_targets[cycle]}", parent=build)
            for content in entry.contents:
                Node(re.findall(r"alt=\"(.*)\" c", str(content))[0], early)
        # Core Build
        if cycle == 2 and verbose:
            core = Node(f"Core Items {time_targets[cycle]}", parent=build)
            for content in entry.contents:
                Node(re.findall(r"alt=\"(.*)\" c", str(content))[0], core)
        # Full Build
        if cycle == 3:
            if verbose:
                full = Node(f"Final Items", parent=build)
                for content in entry.contents[-3:]: # only want the last 3
                    Node(re.findall(r"alt=\"(.*)\" c", str(content))[0], full)
            else:
                for content in entry.contents:
                    Node(re.findall(r"alt=\"(.*)\" c", str(content))[0], build)
    
    # Situational items
    if verbose:
        situational = Node("Situational Items", parent=build)
        for entry in soup.find_all("div", class_="css-143dzw8 es5thxd2"):
            Node(re.findall(r"alt=\"(.*)\" c", str(entry.contents[0]))[0], situational)

    # Skill Learn Order
    # for entry in soup.find_all("div", class_="css-70qvj9 ek7zqkr0")[0].contents:
    #     if entry.name == "p":
    #         Node(entry.text, skill)

    # Skill Max Order
    skill = Node("Skill Priority", parent=root)
    # this makes a list of the skill taken at each level
    sequence = []
    for entry in soup.find_all("div", class_="css-1dai7ia eaoplg14"):
        sequence.append(entry.text) 
    # this dict has Q W E keys and values of the level when they get maxed
    max_order = {}
    for ability in ["Q", "W", "E"]:
        max_order[ability] = [i for i, n in enumerate(sequence) if n == ability][-1] # find the last of each
    # iterate over values and copy keys in order of value magnitude
    prio = []
    for i in range (19):
        if i in max_order.values():
            prio.append((list(max_order.keys())[list(max_order.values()).index(i)]))
    for item in prio:
        Node(item, skill)
        
    # return tree
    return root



import time

import typer

from .champion_gg_scraper import champion_gg_scraper
from .mobalytics_scraper import mobalytics_scraper
from .print_tree import print_tree

ggs = typer.Typer()


@ggs.command()
def main(
    champion: str = typer.Argument(..., help="The champion you're playing"),
    role: str = typer.Argument("", help="The role you're playing", show_default=False),
    matchup: str = typer.Option("", "--matchup", "-m", help="Your opposing matchup", show_default=False),
    verbose: bool = typer.Option(False, "--verbose", "-v", help="Show more verbose output"),
    scraper: str = typer.Option("mobalytics.gg", "--scraper", "-s", help="Which site to scrape")
):  

    args = champion, role, matchup, verbose
    start = time.time() # for funsies

    if role.startswith("m"):
        role = "mid"
    elif role.startswith(("a", "b")):
        role = "adc"
    elif role.startswith("j"):
        role = "jungle"
    elif role.startswith("t"):
        role = "top"
    elif role.startswith("s"):
        role = "support"

    if scraper.startswith("m"):
        print_tree(mobalytics_scraper(*args))
    elif scraper.startswith("c"):
        print_tree(champion_gg_scraper(*args))

    print(f"\nFinished in {round(time.time() - start, 3)} s     ✨ glhf ✨")
    input()


# this file exists to support calling the package as 'python -m pip ggs'
# https://typer.tiangolo.com/tutorial/package/#support-python-m-optional

from gg_scrape.argsparse import ggs
ggs(prog_name='ggs')



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gg_scrape']

package_data = \
{'': ['*']}

install_requires = \
['anytree>=2.8.0,<3.0.0',
 'beautifulsoup4>=4.9.3,<5.0.0',
 'requests>=2.25.1,<3.0.0',
 'typer>=0.3.2,<0.4.0']

entry_points = \
{'console_scripts': ['ggs = gg_scrape.argsparse:ggs']}

setup_kwargs = {
    'name': 'gg-scrape',
    'version': '0.1.5',
    'description': 'A little Python CLI app that provides a summary of League champion build information. ',
    'long_description': "# gg-scrape\nA little Python CLI app that provides a League of Legends champion build by scraping the web.\n\nThe goal was to not have to open a browser tab (or be advertised to) to quickly check a build before a match.\nThe HTML is requested and parsed sequentially, so it's somewhat slow (but still faster than opening a browser).\n\n## Installation\n```\npython -m pip install gg-scrape\n```\n\n## Usage\n```\nggs [OPTIONS] CHAMPION [ROLE]\n```\n\n\n\n## Screenshots\n![help text](img/help.PNG)\n![screenshot of the app in use](img/demo.PNG)\n![unavailable build handling](img/default_build.PNG)\n![verbose output](img/verbose_output.PNG)\n\n## Requirements\nDepends on the anytree, beautifulsoup4, typer, and requests Python libraries.\n\n## Contributions\nThanks to [@Mycsina](https://github.com/Mycsina) for feedback and helping to improve and expand this package's functionality!\n\nPull requests are welcome! \n",
    'author': 'Alex W',
    'author_email': 'alex@southsun.tech',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/teauxfu/gg-scrape',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import re

from anytree import Node

from bs4 import BeautifulSoup

import requests


def champion_gg_scraper(champion: str, role: str, matchup: str, verbose: bool) -> Node:
    """Scrapes a build from champion.gg."""

    if role.startswith("m"):
        role = "Middle"
    if matchup != "":
        url = f"https://champion.gg/champion/{champion}/{role}/overview/{matchup}"
        root = Node(
            f"{champion.title()} {role.title()} vs {matchup.title()} from Champion.gg"
        )
    else:
        url = f"https://champion.gg/champion/{champion}/{role}/"
        root = Node(f"{champion.title()} {role.title()} from Champion.gg")
    page = requests.get(url)
    soup = BeautifulSoup(page.content, "html.parser")
    # Parse the webpage
    # Divide webpage into relevant pieces
    if matchup != "":
        relevant_tags = soup.find_all(class_="Inner-sc-7vmxjm-0 cpZSJT")
        items = relevant_tags[2].contents[0].contents[0]
        runes_webpage = relevant_tags[2].contents[0].contents[1]
    else:
        relevant_tags = soup.find_all(class_="Inner-sc-7vmxjm-0 cpZSJT")
        items = relevant_tags[1].contents[0].contents[0]
        runes_webpage = relevant_tags[1].contents[0].contents[1]
    # Create tree nodes
    summoners = Node("Summoner Spells", parent=root)
    build = Node("Build", root)
    starters = Node("Starter Items", build)
    core = Node("Build Path", build)
    final = Node("Final Build", build)
    tree_runes = Node("Runes", root)
    tree_shards = Node("Shards", root)
    # Summoner Spells
    spells = list(
        set(
            re.findall(
                r"Summoner(.*).png", str(items.find_all(class_="sc-fONwsr bsxfkk"))
            )
        )
    )
    for cycle, entry in enumerate(spells):
        # Ignite is referred to as Dot
        if entry == "Dot":
            Node("Ignite", summoners)
            spells[cycle] = "Ignite"
        else:
            Node(entry, summoners)
    # Starting items
    for entry in items.contents[0].find_all("img"):
        if entry.attrs["alt"] != "":
            Node(entry.attrs["alt"], starters)
    # Core build path
    for entry in items.contents[1].find_all("img"):
        Node(entry.attrs["alt"], core)
    # Final build
    for entry in items.contents[2].find_all("img"):
        Node(entry.attrs["alt"], final)
    # Runes and shards
    runes = runes_webpage.find_all(champion_gg_runes)
    for entry in runes[:6]:
        Node(entry.contents[0], tree_runes)
    for entry in runes[6:]:
        Node(entry.contents[0], tree_shards)
    return root


def champion_gg_runes(tag):
    """Filter function that returns true for tags with the names of Runes in Champion.gg"""
    if "class" in tag.attrs:
        if "ChampionRuneSmallCHGG__RuneName-sc-1vubct9-5" in tag.attrs[
            "class"
        ] and not re.compile("sc-").search(tag.attrs["class"][1]):
            return tag
